import abc
import requests
import logging
from response_measurer.calculaters import *


class Methods:
    def __init__(self):
        """
        TODO: fill here
        """

    @staticmethod
    @abc.abstractmethod
    def send_request(parameters: dict):
        """
        Implements the HTTP request based on the type
        :return: numerical list
        """

    @staticmethod
    def calculate(values):
        # For now, we are just calculating values for one row.
        # values_list = []
        values_dict = {}
        # Calculate mean
        mean = calculate_mean(values)
        values_dict.update(
            {
                "mean": mean
            }
        )
        # Percentile %50
        p50 = calculate_percentile(values, 0.5)
        values_dict.update(
            {
                "P50": p50
            }
        )
        # Percentile %99
        p99 = calculate_percentile(values, 0.99)
        values_dict.update(
            {
                "P99": p99
            }
        )
        return values_dict



import requests
import logging
from response_measurer.methods import Methods


class Get(Methods):
    def __init__(self):
        super(Get, self).__init__()

    @staticmethod
    def send_request(parameters: dict):
        logging.warning("GET method is not implemented")
        exit(1)
        try:
            logging.debug("GET request started")
            values_list = []
            for i in range(parameters["loop_count"]):
                # TODO: To be done
                print("TODO")
            logging.debug("GET request finished")
            return Methods.calculate(values_list)
        except Exception as e:
            logging.error(e)
            exit(1)



import csv
import logging

# TODO: Define different output methods


class PrintOut:
    def __init__(self):
        """
        TODO: fill here
        """

    @staticmethod
    def extract_headers(datas):
        return list(datas.keys())


class PrintCsv(PrintOut):
    def __init__(self):
        super(PrintCsv, self).__init__()

    @staticmethod
    def print(datas):
        field_names = PrintOut.extract_headers(datas)
        with open('hede.csv', 'w', encoding='UTF8') as f:
            writer = csv.DictWriter(f, fieldnames=field_names)
            writer.writeheader()
            datas_list = [datas]
            writer.writerows(datas_list)
        logging.info("Result is printed as CSV file")


class PrintTerminal(PrintOut):
    def __init__(self):
        super(PrintTerminal, self).__init__()

    @staticmethod
    def print(datas):
        logging.info(f"{datas}")



import requests
import logging
from response_measurer.methods import Methods


class Post(Methods):
    def __init__(self):
        super(Post, self).__init__()

    @staticmethod
    def send_request(parameters: dict):
        try:
            logging.debug("POST request started")
            values_list = []
            for i in range(parameters["loop_count"]):
                response = requests.post(
                    parameters["host"],
                    data=parameters["data"],
                    timeout=parameters["timeout"]
                )
                values_list.append(response.elapsed.total_seconds())
            logging.debug("POST request finished")
            return Methods.calculate(values_list)
        except Exception as e:
            logging.error(e)
            exit(1)
